/***************************************************************************
 *   Copyright (C) 2010 by Naomasa Matsubayashi   *
 *   fadis@quaternion.sakura.ne.jp   *
 *                                                                         *
 *   All rights reserved.                                                  *
 *                                                                         *
 * Redistribution and use in source and binary forms, with or without      *
 * modification, are permitted provided that the following conditions are  *
 * met:                                                                    *
 *                                                                         *
 *  1. Redistributions of source code must retain the above copyright      *
 *     notice, this list of conditions and the following disclaimer.       *
 *  2. Redistributions in binary form must reproduce the above copyright   *
 *     notice, this list of conditions and the following disclaimer in the *
 *     documentation and/or other materials provided with the distribution.*
 *                                                                         *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ***************************************************************************/

#include <k6x8/conv.h>
#include <k6x8/map.h>

// EUCJPから16bit内部文字コードに変換する
// 改行が見つかった場合はそこで処理を終了し改行コードの次の文字の先頭アドレスを返す
// ヌル文字が見つかった場合はそこで処理を終了し、ヌル文字を指すアドレスを返す
const char *fstEUCJPToInternal( uint16_t *_buffer, unsigned int _size, const char *_string ) {
  unsigned int cur;
  // 現在のカーソル位置が右端に達するか、あるいはヌル文字が現れるまで
  for( cur = 0; cur != _size - 1 && *_string; cur++ ) {
    // UNIX/Windowsスタイル改行コードを見つけたら
    if( _string[ 0 ] == 0x0A ) {
      // Windowsスタイル改行コードの場合改行コードが2バイトなので次のバイトを読み飛ばす
      if( _string[ 1 ] == 0x0D )
        _string += 2;
      else
        _string++;
      break;
    }
    // MacOSスタイルの改行コードを見つけたら
    else if( *_string == 0x0D ) {
      _string++;
      break;
    }
    // それ以外の文字だったら
    else {
      // 描く
      _buffer[ cur ] = fstGetGlyphIndex( (const uint8_t*)_string );
      _string = (const char *)fstNextCharcter( (const uint8_t*)_string );
    }
  }
  _buffer[ cur ] = (uint16_t)-1;
  return _string;
}
